#pragma once
#include <ChamberComponent.h>
#include <SpecialCell.h>
#include <memory>
#include <iostream>

class Layer : public ChamberComponent {
public:
  Layer(ChamberComponent* parent, const ColourStrategy* colourStrategy,
        const WidthStrategy* widthStrategy = new WidthStrategyOne()) :
    ChamberComponent(parent),
    m_widthStrategy(widthStrategy),
    m_colourStrategy(colourStrategy)
  {
  }

  void fillCells() override
  {
    unsigned width = m_widthStrategy->getWidth();
    for (int ii = m_children.size() * m_widthStrategy->getWidth(); ii < getMaxX(); ii += width) {
      m_children.emplace_back(new SpecialCell(ii, getMyY(), this, m_colourStrategy, m_widthStrategy));

    }
  }

  /** By definition a layer has hight 1.
   *
   *  Have a look into the visualize function to see the truth of this statement.
   */
  unsigned getMaxY() const override
  {
    return 1;
  }

  /** Overrides to draw some part of the y-axis before drawing all the Cells in one line.
   */
  std::string visualize() override
  {
    std::cout <<  getMyY() << "\t | ";
    for (auto& childPtr : m_children) {
      std::cout << childPtr->visualize();
    }
    std::cout << std::endl;
    return "";
  }

private:
  const WidthStrategy*    m_widthStrategy;
  const ColourStrategy*   m_colourStrategy;
};
