#pragma once

#include <ChamberComponent.h>
#include <Layer.h>
#include <ColourStrategy.h>
#include <WidthStrategy.h>

#include <memory>

/** SuperLayers are the Components below the complete CompositeChamber and above the single Layers.
 *
 *  The ColourStrategy is defined by the SuperLayer, e.g. all Layers in a SuperLayer have the same
 *  colour.
 */
class SuperLayer : public ChamberComponent {
public:
  /** Constructor assigning a CompositeChamber as parent and a ColourStrategy.
   */
  SuperLayer(ChamberComponent* parent, ColourStrategy const* colourStrategy = new ColourStrategyBlue()) :
    ChamberComponent(parent),
    m_colourStrategy(colourStrategy)
  {}

  /** Adding a Layer as daughter to the SuperLayer and defining the width of each Cell in the Layer.
   *
   *  All Cells in a Layer have the same width.
   */
  Layer*  addLayer(const WidthStrategy* widthStrategy = new WidthStrategyOne())
  {
    m_children.emplace_back(new Layer(this, m_colourStrategy, widthStrategy));
    return dynamic_cast<Layer*>(m_children.back());
  }

private:
  const ColourStrategy*   m_colourStrategy;
};
