#include <BackgroundVisitor.h>
#include <CompositeChamber.h>
#include <ColourStrategy.h>
#include <WidthStrategy.h>
#include <Particle.h>

#include <vector>
#include <memory>
#include <iostream>
#include <random>
#include <cmath>

using namespace std;

int main()
{
  cout << "Creating a composite practicePolicyProject!" << endl;

  // Config info:
  unsigned xSize = 100;
  std::vector<unsigned> nLayers = {8, 6, 6, 6, 6};

  //Building of the Composite.
  CompositeChamber compositeChamber(xSize);
  for (int ii = 0; ii < nLayers.size(); ii++) {
    SuperLayer* superLayer = 0;
    if (ii % 2) {
      superLayer = compositeChamber.addSuperLayer(new ColourStrategyBlue());
    } else {
      superLayer = compositeChamber.addSuperLayer(new ColourStrategyRed());
    }

    for (int jj = 0; jj < nLayers[ii]; ++jj) {
      if (jj < nLayers[ii] / 2) {
        superLayer->addLayer(new WidthStrategyOne());
      } else {
        superLayer->addLayer(new WidthStrategyTwo());
      }
    }
  }
  compositeChamber.fillCells();

  cout << "Maximum X of the Chamber: " << compositeChamber.getMaxX() << endl
       << "Maximum Y of the Chamber: " << compositeChamber.getMaxY() << endl;

  //Adding Background
  BackgroundVisitor backgroundVisitor;
  for (auto chamberIterator = compositeChamber.first();
       chamberIterator.current() != nullptr; ++chamberIterator) {
    chamberIterator.current()->accept(backgroundVisitor);
  }

  //Adding a Track
  srand(time(NULL));
  float pX = (rand() / static_cast<float>(RAND_MAX) - 0.5) * 2.;
  float pY =  rand() / static_cast<float>(RAND_MAX);

  //Lets limit the the x-position of the particle to the middle half.
  float chamberSizeX = static_cast<float>(compositeChamber.getMaxX());
  float x  = (((rand() / static_cast<float>(RAND_MAX)) * chamberSizeX) + (chamberSizeX / 2.)) / 2.;

  Particle particle(pX, pY, x);

  while (particle.getXPosition() < compositeChamber.getMaxX() && particle.getYPosition() < compositeChamber.getMaxY()) {
    //find nearest cell
    int xPosition = static_cast<unsigned>(particle.getXPosition());
    int yPosition = static_cast<unsigned>(particle.getYPosition());

    Cell* myCellPtr = compositeChamber.getCellAt(xPosition, yPosition);
    if (myCellPtr) {
      myCellPtr->addEDeposition(1);
    }

    particle.makeStep();
  }

  cout << "Visualizing the Chamber after a Track passed." << endl
       << "x    : " << x  << endl
       << "Angle: " << acos(pX / sqrt(pow(pY, 2) + pow(pX, 2))) * (180. / 3.14) << endl;

  compositeChamber.visualize();
}

